/**
 * @file generateAcconfig.c acconfig.h generation
 * 
 * $Id: generate_acconfig.c,v 1.8 2001/12/31 09:26:54 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

void
generateAcconfig(ProjectInfo *project, GenerationState *state)
{
	char *filename;
	char *ucName;
	char *temp;
	FILE *f;

	Package *pkg;

	fprintf(stderr, "[%s]\n", project->name);
	
	updateGenerationStatus(state, "Creating:\n\t", "acconfig.h");

	filename = (char *)malloc((strlen(project->dir) + 12) * sizeof(char));
	
	strcpy(filename, project->dir);
	strcat(filename, "/acconfig.h");

	if ((f = fopen(filename, "w")) == NULL)
	{
		fprintf(stderr, "Could not write to %s\n", filename);
		exit(1);
	}

	free(filename);

	fprintf(stderr, "[%s]\n", project->name);

	temp = stringToUpper(project->name);
	ucName = fixProjectName(temp);
	free(temp);

	fprintf(f, "/* Generated by genst */\n\n");
	fprintf(f, "#undef %s_MAJOR_VERSION\n", ucName);
	fprintf(f, "#undef %s_MINOR_VERSION\n", ucName);
	fprintf(f, "#undef %s_MICRO_VERSION\n", ucName);
	fprintf(f, "#undef %s_VERSION\n", ucName);

	for (pkg = project->packages; pkg != NULL; pkg = pkg->next)
	{
		Check *check;

		for (check = pkg->checks; check != NULL; check = check->next)
		{
			if ((check->define != NULL) && (!isWhitespace(check->define)))
				fprintf(f, "#undef %s\n", check->define);
		}
	}

	fclose(f);

	free(ucName);
}

