/**
 * @file generateM4.c projectname.m4 generation
 *
 * $Id: generate_m4.c,v 1.7 2001/12/31 09:26:55 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

void
generateM4(ProjectInfo *project, GenerationState *state)
{
	char *filename;
	char *ucName;
	char *temp;
	FILE *f;

	if (project->target != TARGET_BOTH && project->target != TARGET_LIBRARY)
		return;

	filename = (char *)malloc((strlen(project->name) + 7) * sizeof(char));
	
	sprintf(filename, "%s.m4.in", project->name);
	updateGenerationStatus(state, "Creating:\n\t", filename);

	free(filename);
	
	filename = (char *)malloc((strlen(project->name) + strlen(project->dir) +
							   8) * sizeof(char));

	sprintf(filename, "%s/%s.m4.in", project->dir, project->name);

	if ((f = fopen(filename, "w")) == NULL)
	{
		fprintf(stderr, "Could not write to %s\n", filename);
		exit(1);
	}

	free(filename);

	temp = stringToUpper(project->name);
	ucName = fixProjectName(temp);
	free(temp);

	fprintf(f, "dnl # $" "Id" "$\n\n");
	fprintf(f, "dnl # Configure paths for lib%s\n\n", project->name);

	fprintf(f, "dnl AM_PATH_%s([MINIMUM-VERSION, [ACTION-IF-FOUND"
			   " [, ACTION-IF-NOT-FOUND]]])\n", ucName);
	fprintf(f, "dnl Test for %s, and define %s_CFLAGS and %s_LIBS\n",
			ucName, ucName, ucName);
	fprintf(f, "dnl\n");
	
	fprintf(f, "AC_DEFUN(AM_PATH_%s,\n", ucName);
	fprintf(f, "[dnl\n");
	fprintf(f, "dnl Get the cflags and libraries from the %s-config script\n",
			project->name);
	fprintf(f, "dnl\n");
	
	fprintf(f, "AC_ARG_WITH(%s-prefix,[  --with-%s-prefix=PFX"
			   "  Prefix where %s is installed (optional)],\n",
			   project->name, project->name, project->name);
	fprintf(f, "\t\t%s_config_prefix=\"$withval\", %s_config_prefix=\"\")\n",
			project->name, project->name);
	fprintf(f, "AC_ARG_WITH(%s-exec-prefix,[  --with-%s-exec-prefix=PFX"
			   "  Exec prefix where %s is installed (optional)],\n",
			   project->name, project->name, project->name);
	fprintf(f, "\t\t%s_config_exec_prefix=\"$withval\","
			   " %s_config_exec_prefix=\"\")\n",
			   project->name, project->name);
	fprintf(f, "AC_ARG_ENABLE(%stest, [  --disable-%stest"
			   "  Do not try to compile and run a test %s program],\n",
			   project->name, project->name, project->name);
	fprintf(f, "\t\t, enable_%stest=yes)\n\n", project->name);
	fprintf(f, "\tif test x$%s_config_exec_prefix != x ; then\n",
			project->name);
	fprintf(f, "\t\t%s_config_args=\"$%s_config_args --exec-prefix="
			   "$%s_config_exec_prefix\"\n",
			   project->name, project->name, project->name);
	fprintf(f, "\t\tif test x${%s_CONFIG+set} != xset ; then\n", ucName);
	fprintf(f, "\t\t\t%s_CONFIG=$%s_config_exec_prefix/bin/%s-config\n",
			ucName, project->name, project->name);
	fprintf(f, "\t\tfi\n");
	fprintf(f, "\tfi\n");
	fprintf(f, "\tif test x$%s_config_prefix != x ; then\n", project->name);
	fprintf(f, "\t\t%s_config_args=\"$%s_config_args"
			   " --prefix=$%s_config_prefix\"\n",
			   project->name, project->name, project->name);
	fprintf(f, "\t\tif test x${%s_CONFIG+set} != xset ; then\n", ucName);
	fprintf(f, "\t\t\t%s_CONFIG=$%s_config_prefix/bin/%s-config\n",
			ucName, project->name, project->name);
	fprintf(f, "\t\tfi\n");
	fprintf(f, "\tfi\n\n");

	fprintf(f, "\tAC_PATH_PROG(%s_CONFIG, %s-config, no)\n\n",
			ucName, project->name);
	
	fprintf(f, "\tmin_%s_version=ifelse([$1], , @VERSION@, $1)\n",
			project->name);
	fprintf(f, "\tAC_MSG_CHECKING([for %s - version >= $min_%s_version])\n",
			project->name, project->name);
	fprintf(f, "\tno_%s=\"\"\n", project->name);
	fprintf(f, "\tif test \"$%s_CONFIG\" = \"no\" ; then\n", ucName);
	fprintf(f, "\t\tno_%s=yes\n", project->name);
	fprintf(f, "\telse\n");
	fprintf(f, "\t\t%s_CFLAGS=`$%s_CONFIG $%s_config_args --cflags`\n",
			ucName, ucName, project->name);
	fprintf(f, "\t\t%s_LIBS=`$%s_CONFIG $%s_config_args --libs`\n",
			ucName, ucName, project->name);
	fprintf(f, "\t\t%s_config_major_version=`$%s_CONFIG $%s_config_args"
			   " --version | \\\n", project->name, ucName, project->name);
	fprintf(f, "\t\t\tsed 's/[[^0-9]]*\\([[0-9]]*\\).\\([[0-9]]*\\)."
			   "\\([[0-9]]*\\)/\\1/'`\n");
	fprintf(f, "\t\t%s_config_minor_version=`$%s_CONFIG $%s_config_args"
			   " --version | \\\n", project->name, ucName, project->name);
	fprintf(f, "\t\t\tsed 's/[[^0-9]]*\\([[0-9]]*\\).\\([[0-9]]*\\)."
			   "\\([[0-9]]*\\)/\\2/'`\n");
	fprintf(f, "\t\t%s_config_micro_version=`$%s_CONFIG $%s_config_args"
			   " --version | \\\n", project->name, ucName, project->name);
	fprintf(f, "\t\t\tsed 's/[[^0-9]]*\\([[0-9]]*\\).\\([[0-9]]*\\)."
			   "\\([[0-9]]*\\)/\\3/'`\n");
	
	fprintf(f, "\t\tif test \"x$enable_%stest\" = \"xyes\" ; then\n",
			project->name);
	fprintf(f, "\t\t\tac_save_CFLAGS=\"$CFLAGS\"\n");
	fprintf(f, "\t\t\tac_save_LIBS=\"$LIBS\"\n");
	fprintf(f, "\t\t\tCFLAGS=\"$CFLAGS $%s_CFLAGS\"\n", ucName);
	fprintf(f, "\t\t\tLIBS=\"$%s_LIBS $LIBS\"\n", ucName);

	fprintf(f, "dnl\n");
	fprintf(f, "dnl Now check if the installed %s is sufficiently new.\n",
			project->name);
	fprintf(f, "dnl (Also sanity checks the results of %s-config to some"
			   " extent.)\n", project->name);
	fprintf(f, "dnl\n");
	fprintf(f, "\t\t\trm -f conf.%stest\n", project->name);
	fprintf(f, "\t\t\tAC_TRY_RUN([\n");
	
	fprintf(f, "#include <%s-config.h>\n", project->name);
	fprintf(f, "#include <stdio.h>\n");
	fprintf(f, "#include <stdlib.h>\n");
	fprintf(f, "#include <string.h>\n\n");
	fprintf(f, "char *my_strdup(char *str)\n");
	fprintf(f, "{\n");
	fprintf(f, "\tchar *newStr;\n\n");
	fprintf(f, "\tif (str)\n");
	fprintf(f, "\t{\n");
	fprintf(f, "\t\tnewStr = (char *)malloc((strlen(str) + 1)"
			   " * sizeof(char));\n");
	fprintf(f, "\t\tstrcpy(newStr, str);\n");
	fprintf(f, "\t}\n");
	fprintf(f, "\telse\n");
	fprintf(f, "\t\tnewStr = NULL;\n\n");
	fprintf(f, "\treturn newStr;\n");
	fprintf(f, "}\n\n");
	
	fprintf(f, "int main()\n");
	fprintf(f, "{\n");
	fprintf(f, "\tint major, minor, micro;\n");
	fprintf(f, "\tchar *tmp_version;\n\n");
	fprintf(f, "\tsystem(\"touch conf.%stest\");\n\n", project->name);
	fprintf(f, "\t/* HP/UX 9 (%%@#!) writes to sscanf strings */\n");
	fprintf(f, "\ttmp_version = my_strdup(\"$min_%s_version\");\n",
			project->name);
	fprintf(f, "\tif (sscanf(tmp_version, \"%%d.%%d.%%d\", &major, &minor,"
			   "&micro) != 3) {\n");
	fprintf(f, "\t\tprintf(\"%%s, bad version string\\n\","
			   " \"$min_%s_version\");\n", project->name);
	fprintf(f, "\t\texit(1);\n");
	fprintf(f, "\t}\n\n");
	
	fprintf(f, "\tif (($%s_config_major_version  > major) ||\n",
			project->name);
	fprintf(f, "\t   (($%s_config_major_version == major) &&"
			   " ($%s_config_minor_version  > minor)) ||\n",
			   project->name, project->name);
	fprintf(f, "\t   (($%s_config_major_version == major) &&"
			   " ($%s_config_minor_version == minor) &&\n",
			   project->name, project->name);
	fprintf(f, "\t    ($%s_config_micro_version >= micro)))\n", project->name);
	fprintf(f, "\t{\n");
	fprintf(f, "\t\treturn 0;\n");
	fprintf(f, "\t}\n");
	fprintf(f, "\telse\n");
	fprintf(f, "\t{\n");
	fprintf(f, "\t\tprintf(\"\\n*** '%s-config --version' returned "
			   "%%d.%%d.%%d, but %s (%%d.%%d.%%d)\\n\",\n",
			   project->name, project->name);
	fprintf(f, "\t\t       $%s_config_major_version, $%s_config_minor_version,"
			   " $%s_config_micro_version,\n",
			   project->name, project->name, project->name);
	fprintf(f, "\t\t       major, minor, micro);\n");
	fprintf(f, "\t\tprintf(\"*** was found! If %s-config was correct,"
			   " then it is best\\n\");\n", project->name);
	fprintf(f, "\t\tprintf(\"*** to remove the old version of %s. You may"
			   " also be able to fix the error\\n\");\n", project->name);
	fprintf(f, "\t\tprintf(\"*** by modifying your LD_LIBRARY_PATH"
			   " environment variable, or by editing\\n\");\n");
	fprintf(f, "\t\tprintf(\"*** /etc/ld.so.conf. Make sure you have run"
			   " ldconfig if that is\\n\");\n");
	fprintf(f, "\t\tprintf(\"*** required on your system.\\n\");\n");
	fprintf(f, "\t\tprintf(\"*** If %s-config was wrong, set the"
			   " environment variable %s_CONFIG\\n\");\n", project->name,
			   ucName);
	fprintf(f, "\t\tprintf(\"*** to point to the correct copy of %s-config,"
			   " and remove the file config.cache\\n\");\n", project->name);
	fprintf(f, "\t\tprintf(\"*** before re-running configure.\\n\");\n");
	fprintf(f, "\t\treturn 1;\n");
	fprintf(f, "\t}\n");
	fprintf(f, "}\n");
	fprintf(f, "],, no_%s=yes,[echo $ac_n \"cross compiling; assumed OK..."
			   " $ac_c\"])\n", project->name);
	fprintf(f, "\t\t\tCFLAGS=\"$ac_save_CFLAGS\"\n");
	fprintf(f, "\t\t\tLIBS=\"$ac_save_LIBS\"\n");
	fprintf(f, "\t\tfi\n");
	fprintf(f, "\tfi\n");
	
	fprintf(f, "\tif test \"x$no_%s\" = x ; then\n", project->name);
	fprintf(f, "\t\tAC_MSG_RESULT(yes)\n");
	fprintf(f, "\t\tifelse([$2], , :, [$2])\n");
	fprintf(f, "\telse\n");
	fprintf(f, "\t\tAC_MSG_RESULT(no)\n");
	fprintf(f, "\t\tif test \"$%s_CONFIG\" = \"no\" ; then\n", ucName);
	fprintf(f, "\t\t\techo \"*** The %s-config script installed by %s"
			   " could not be found\"\n", project->name, project->name);
	fprintf(f, "\t\t\techo \"*** If %s was installed in PREFIX, make sure"
			   " PREFIX/bin is in\"\n", project->name);
	fprintf(f, "\t\t\techo \"*** your path, or set the %s_CONFIG"
			   " environment variable to the \"\n", ucName);
	fprintf(f, "\t\t\techo \"*** full path to %s-config.\"\n", project->name);
	fprintf(f, "\t\telse\n");
	fprintf(f, "\t\t\tif test -f conf.%stest ; then\n", project->name);
	fprintf(f, "\t\t\t\t:\n");
	fprintf(f, "\t\t\telse\n");
	fprintf(f, "\t\t\t\techo \"*** Could not run %s test program,"
			   " checking why...\"\n", project->name);
	fprintf(f, "\t\t\t\tCFLAGS=\"$CFLAGS $%s_CFLAGS\"\n", ucName);
	fprintf(f, "\t\t\t\tLIBS=\"$LIBS $%s_LIBS\"\n", ucName);
	fprintf(f, "\t\t\t\tAC_TRY_LINK([\n");
	
	fprintf(f, "#include <%s-config.h>\n", project->name);
	fprintf(f, "#include <stdio.h>\n");
	fprintf(f, "],[\n");
	fprintf(f, "\t\t\t\treturn ((%s_major_version) || (%s_minor_version) ||"
			   " (%s_micro_version));\n",
			   project->name, project->name, project->name);
	fprintf(f, "],[\n");
	fprintf(f, "\t\t\t\techo \"*** The test program compiled, but did not"
			   " run. This usually means\"\n");
	fprintf(f, "\t\t\t\techo \"*** that the run-time linker is not finding"
			   " %s or finding the wrong\"\n", project->name);
	fprintf(f, "\t\t\t\techo \"*** version of %s. If it is not finding %s,"
			   " you'll need to set your\"\n", project->name, project->name);
	fprintf(f, "\t\t\t\techo \"*** LD_LIBRARY_PATH environment variable,"
			   " or edit /etc/ld.so.conf to point\"\n");
	fprintf(f, "\t\t\t\techo \"*** to the installed location. Also, make sure"
			   " you have run ldconfig, if that\"\n");
	fprintf(f, "\t\t\t\techo \"*** is required on your system.\"\n");
	fprintf(f, "\t\t\t\techo \"***\"\n");
	fprintf(f, "\t\t\t\techo \"*** If you have an old version installed,"
			   " it is best to remove it, although\"\n");
	fprintf(f, "\t\t\t\techo \"*** you may also be able to get things to"
			   " work by modifying LD_LIBRARY_PATH.\"\n");
	
	fprintf(f, "\t\t\t\t])\n");
	fprintf(f, "\t\t\t\tCFLAGS=\"$ac_save_CFLAGS\"\n");
	fprintf(f, "\t\t\t\tLIBS=\"$ac_save_LIBS\"\n");
	fprintf(f, "\t\t\tfi\n");
	fprintf(f, "\t\tfi\n");
	fprintf(f, "\t\t%s_CFLAGS=\"\"\n", ucName);
	fprintf(f, "\t\t%s_LIBS=\"\"\n", ucName);
	fprintf(f, "\t\tifelse([$3], , :, [$3])\n");
	fprintf(f, "\tfi\n");
	fprintf(f, "\tAC_SUBST(%s_CFLAGS)\n", ucName);
	fprintf(f, "\tAC_SUBST(%s_LIBS)\n", ucName);
	fprintf(f, "\trm -f conf.%stest\n", project->name);
	fprintf(f, "])\n");
	
	fclose(f);

	free(ucName);
}

