/**
 * @file generateRpmSpec.c projectname.spec.in generation
 * 
 * $Id: generate_spec.c,v 1.6 2001/12/31 09:26:55 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

void
generateRpmSpec(ProjectInfo *project, GenerationState *state)
{
	char *filename;
	FILE *f;
	
	filename = (char *)malloc((strlen(project->name) + 9) * sizeof(char));

	sprintf(filename, "%s.spec.in", project->name);
	updateGenerationStatus(state, "Creating:\n\t", filename);

	free(filename);

	filename = (char *)malloc((strlen(project->dir) + strlen(project->name) +
							   10) * sizeof(char));

	sprintf(filename, "%s/%s.spec.in", project->dir, project->name);

	if ((f = fopen(filename, "w")) == NULL)
	{
		fprintf(stderr, "Could not write to %s\n", filename);
		exit(1);
	}

	free(filename);

	fprintf(f, "# $" "Id" "$\n\n");
	fprintf(f, "%%define RELEASE 1\n");
	fprintf(f, "%%define rel     %%{?CUSTOM_RELEASE}"
			   " %%{!?CUSTOM_RELEASE:%%RELEASE}\n");
	fprintf(f, "%%define prefix  @prefix@\n\n");

	if (project->target == TARGET_BOTH || project->target == TARGET_EXECUTABLE)
	{
		fprintf(f, "Summary:       %s\n", project->name);
		fprintf(f, "Name:          @PACKAGE@\n");
		fprintf(f, "Version:       @VERSION@\n");
		fprintf(f, "Release:       %%rel\n");
		fprintf(f, "Copyright:     %s\n", project->license);
		fprintf(f, "Group:         %s\n", project->group);
		fprintf(f, "Source:        \n");
		fprintf(f, "URL:           \n");
		fprintf(f, "Prefix:        %%prefix\n");
		fprintf(f, "BuildRoot:     /var/tmp/%%{name}-%%{version}-root\n");
		fprintf(f, "Packager:      \n\n");
		fprintf(f, "%%description\n\n\n");
	}

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "%%package      devel\n");
		fprintf(f, "Summary:       %s\n", project->summary);
		fprintf(f, "Group:         %s\n", project->group);
		fprintf(f, "%%description  devel\n\n\n");
	}

	fprintf(f, "%%prep\n");
	fprintf(f, "%%setup\n\n");
	fprintf(f, "%%build\n");
	fprintf(f, "if [ ! -z \"`echo -n ${RPM_OPT_FLAGS} | grep pentium`\"];"
			   " then\n");
	fprintf(f, "\tif [ ! -z `which egcs` ]; then\n");
	fprintf(f, "\t\tCCC=\"egcs\"\n");
	fprintf(f, "\telse\n");
	fprintf(f, "\t\tif [ ! -z `which pgcc` ]; then\n");
	fprintf(f, "\t\t\tCC=\"pgcs\"\n");
	fprintf(f, "\t\tfi\n");
	fprintf(f, "\tfi\n");
	fprintf(f, "\tCFLAGS=\"${RPM_OPT_FLAGS}\"\n");
	fprintf(f, "else\n");
	fprintf(f, "\tCFLAGS=\"${RPM_OPT_FLAGS}\"\n");
	fprintf(f, "fi\n\n");
	
	fprintf(f, "if [ ! -f configure ]; then\n");
	fprintf(f, "\tCFLAGS=\"$RPM_OPT_FLAGS\" ./autogen.sh --prefix=%%prefix\n");
	fprintf(f, "else\n");
	fprintf(f, "\tCFLAGS=\"$RPM_OPT_FLAGS\" ./configure --prefix=%%prefix\n");
	fprintf(f, "fi\n\n");

	fprintf(f, "make\n\n");
	
	fprintf(f, "%%install\n");
	fprintf(f, "rm -rf $RPM_BUILD_ROOT\n");
	fprintf(f, "make DESTDIR=$RPM_BUILD_ROOT install\n\n");

	fprintf(f, "%%clean\n");
	fprintf(f, "rm -rf $RPM_BUILD_ROOT\n\n");

	fprintf(f, "%%post\n\n");
	fprintf(f, "%%postun\n\n");
	fprintf(f, "%%files\n");
	fprintf(f, "%%defattr(-, root, root)\n\n");

	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "%%{prefix}/lib/lib*.so.*\n");
		fprintf(f, "%%{prefix}/lib/*.la\n");
		fprintf(f, "%%{prefix}/lib/*.so\n");
	}

	if (project->target == TARGET_BOTH || project->target == TARGET_EXECUTABLE)
	{
		fprintf(f, "%%{prefix}/bin/*\n");
		fprintf(f, "%%{prefix}/share/*\n");
	}

	fprintf(f, "\n");
	
	fprintf(f, "%%doc AUTHORS COPYING ChangeLog NEWS README\n\n");
	
	if (project->target == TARGET_BOTH || project->target == TARGET_LIBRARY)
	{
		fprintf(f, "%%files devel\n");
		fprintf(f, "%%defattr(-, root, root)\n\n");
		fprintf(f, "%%{prefix}/include/*.h\n");
		fprintf(f, "%%{prefix}/share/aclocal/%s.m4\n", project->name);
	}
	
	fclose(f);
}

