/**
 * @file package.h Package information
 *
 * $Id: package.h,v 1.7 2001/12/31 09:26:55 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public List
 * as published by the Free Software Foundation; either version 2
 * of the List, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public List for more details.
 *
 * You should have received a copy of the GNU General Public List
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _GENST_PACKAGE_H_
#define _GENST_PACKAGE_H_

#include "genst.h"
/* #include "dialog.h" */

typedef enum
{
	PACKAGE_NONE,
	PACKAGE_NAME,
	PACKAGE_DESCRIPTION,
	PACKAGE_FILE,
	PACKAGE_MACRO,
	PACKAGE_DEFINE,
	PACKAGE_CFLAGS,
	PACKAGE_LIBS

} PackageTags;

typedef struct _Check Check;
struct _Check
{
	char *name;
	char *description;
	char *file;
	char *macro;
	char *define;
	char *cflags;
	char *libs;

	Check *next;
	Check *prev;
};

typedef struct _Package Package;
struct _Package
{
	char *name;
	char *description;

	Check *checks;
	Check *lastCheck;

	Package *next;
	Package *prev;
};

typedef struct
{
	PackageTags tag;

	char inInfo;
	char inChecks;
	
	DialogList *list;
	Package    *package;
	Check      *check;

	char **defaults;
	int defaultCount;

} PackageParserState;


/**************************************************************************
 * Package functions
 **************************************************************************/
Package *newPackage();
Package *loadPackage(const char *packageName, DialogList *list,
					 char **defaults, int defaultCount);
void freePackage(Package *pkg);
Package *copyPackage(Package *src);

int packageToArray(Package *pkg, char ***array);

/**************************************************************************
 * Check functions
 **************************************************************************/
Check *newCheck();
void freeCheck(Check *check);
Check *getCheckWithName(Package *pkg, const char *name);
Check *getFirstCheck(Package *pkg);
Check *getNextCheck(Check *check);
Check *copyCheck(Check *src);
int getCheckCount(Package *pkg);
void addCheck(Package *pkg, Check *check);

#endif /* _GENST_PACKAGE_H_ */
