/**
 * @file packagelist.c Package list XML parser
 * 
 * $Id: packagelist.c,v 1.11 2002/10/21 19:15:40 mhearn Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

#include <libxml/parser.h>

static void
__startElement(void *userData, const char *name, const char **attrs)
{
	PackageListParserState *state = (PackageListParserState *)userData;
	
	if      (!strcmp(name, "package"))     state->tag = PKGLIST_NONE;
	else if (!strcmp(name, "name"))        state->tag = PKGLIST_NAME;
	else if (!strcmp(name, "description")) state->tag = PKGLIST_DESCRIPTION;
}

static void
__endElement(void *userData, const char *name)
{
	PackageListParserState *state = (PackageListParserState *)userData;

	if (!strcmp(name, "package"))
	{
		MenuItem *item;
		
		if (state->name == NULL)
		{
			fprintf(stderr, "XML Error: package must have a name!\n");
			exit(1);
		}
		
		item = newDialogMenuItem(state->name, state->description);
		dialogAddMenuItem(state->menu, item);

		free(state->name);
		state->name = NULL;

		if (state->description != NULL)
		{
			free(state->description);
			state->description = NULL;
		}
	}
	
	state->tag = PKGLIST_NONE;
}

static void
__characters(void *userData, const char *ch, int len)
{
	PackageListParserState *state = (PackageListParserState *)userData;
	
	char *data = (char *)malloc((len + 1) * sizeof(char));

	strncpy(data, ch, len);
	data[len] = '\0';

	if (isWhitespace(data))
	{
		free(data);
		return;
	}

	switch (state->tag)
	{
		case PKGLIST_NONE:
			break;

		case PKGLIST_NAME:
			if (state->name == NULL)
				state->name = strdup(data);
			else
			{
				char *newStr = (char *)malloc((strlen(state->name) +
											   strlen(data) + 1) *
											  sizeof(char));
				
				sprintf(newStr, "%s%s", state->name, data);
				free(state->name);
				state->name = newStr;
			}
			break;

		case PKGLIST_DESCRIPTION:
			if (state->description == NULL)
				state->description = strdup(data);
			else
			{
				char *newStr = (char *)malloc((strlen(state->description) +
											   strlen(data) + 1) *
											  sizeof(char));
				
				sprintf(newStr, "%s%s", state->description, data);
				free(state->description);
				state->description = newStr;
			}
			break;
	}

	free(data);
}

static xmlSAXHandler pkglistHandler = {
	NULL,                                     /* internalSubset        */
	NULL,                                     /* isStandalone          */
	NULL,                                     /* hasInternalSubset     */
	NULL,                                     /* hasExternalSubset     */
	NULL,                                     /* resolveEntity         */
	NULL,                                     /* getEntity             */
	NULL,                                     /* entityDecl            */
	NULL,                                     /* notationDecl          */
	NULL,                                     /* attributeDecl         */
	NULL,                                     /* elementDecl           */
	NULL,                                     /* unparsedEntityDecl    */
	NULL,                                     /* setDocumentLocator    */
	NULL,                                     /* startDocument         */
	NULL,                                     /* endDocument           */
	(startElementSAXFunc)__startElement,      /* startElement          */
	(endElementSAXFunc)__endElement,          /* endElement            */
	NULL,                                     /* reference             */
	(charactersSAXFunc)__characters,          /* characters            */
	NULL,                                     /* ignorableWhitespace   */
	NULL,                                     /* processingInstruction */
	NULL,                                     /* comment               */
	(warningSAXFunc)xmlWarning,              /* warning               */
	(errorSAXFunc)xmlError,                  /* error                 */
	(fatalErrorSAXFunc)xmlFatalError        /* fatalError            */
};

void
addPackages(DialogMenu *menu)
{
	PackageListParserState state;
	char *file;

	file = (char *)malloc((strlen(data_dir) + 14) * sizeof(char));
	
	strcpy(file, data_dir);
	strcat(file, "/packages.xml");

	state.menu        = menu;
	state.name        = NULL;
	state.description = NULL;
	
	parseXml(&pkglistHandler, &state, file);

	free(file);
}

